@@ -1,6 +1,6 @@
 /* exif-ifd.h
  *
- * Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
+ * Copyright (c) 2002 Lutz Mueller <lutz@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,15 +21,23 @@
 #ifndef __EXIF_IFD_H__
 #define __EXIF_IFD_H__
 
+/*! Possible EXIF Image File Directories */
 typedef enum {
-	EXIF_IFD_0 = 0,
-	EXIF_IFD_1,
-	EXIF_IFD_EXIF,
-	EXIF_IFD_GPS,
-	EXIF_IFD_INTEROPERABILITY,
-	EXIF_IFD_COUNT
+	EXIF_IFD_0 = 0,                /*!< */
+	EXIF_IFD_1,                    /*!< */
+	EXIF_IFD_EXIF,                 /*!< */
+	EXIF_IFD_GPS,                  /*!< */
+	EXIF_IFD_INTEROPERABILITY,     /*!< */
+	EXIF_IFD_COUNT                 /*!< Not a real value, just (max_value + 1). */
 } ExifIfd;
 
+/*! Return a textual name of the given IFD. The name is a short, unique,
+ * non-localized text string containing only US-ASCII alphanumeric
+ * characters.
+ *
+ * \param[in] ifd IFD
+ * \return textual name of the IFD
+ */
 const char *exif_ifd_get_name (ExifIfd ifd);
 
 #endif /* __EXIF_IFD_H__ */
