@@ -22,9 +22,10 @@
 #include <libexif/exif-data.h>
 
 #include <stdio.h>
+#include <stdlib.h>
 
 int
-main (int argc, char **argv)
+main ()
 {
 	ExifData *d;
 	ExifEntry *e;
@@ -33,15 +34,27 @@ main (int argc, char **argv)
 	unsigned int i;
 
 	d = exif_data_new ();
+	if (!d) {
+		printf ("Error running exif_data_new()\n");
+		exit(13);
+	}
+
 	e = exif_entry_new ();
+	if (!e) {
+		printf ("Error running exif_entry_new()\n");
+		exit(13);
+	}
+
 	exif_content_add_entry (d->ifd[EXIF_IFD_0], e);
 	exif_entry_initialize (e, EXIF_TAG_SHUTTER_SPEED_VALUE);
 	exif_set_srational (e->data, exif_data_get_byte_order (d), r);
 
-	for (i = 30; i > 0; i--)
+	for (i = 30; i > 0; i--) {
 		printf ("Length %2i: '%s'\n", i, 
 			exif_entry_get_value (e, v, i));
+	}
 
+	exif_entry_unref (e);
 	exif_data_unref (d);
 
 	return 0;
